"use strict";
var _a, _b, _c;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Apigw = exports.t = void 0;
const common_1 = require("@tencent-serverless-framework/common");
const try2be_1 = require("try2be");
const _ = require("lodash");
const apigateway_client_1 = require("tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_client");
const usage_plan_1 = require("./usage-plan");
const trigger_1 = require("./trigger");
const api_1 = require("./api");
const formatter_1 = require("./formatter");
const custom_domain_1 = require("./custom-domain");
const logger_1 = require("@jsonlog/logger");
const i18next_1 = require("i18next");
const i18next_fs_backend_1 = require("i18next-fs-backend");
const path = require("path");
i18next_1.default.use(i18next_fs_backend_1.default).init({
    initImmediate: false,
    fallbackLng: 'zh',
    lng: (_c = (_b = (_a = process.env.LANG) === null || _a === void 0 ? void 0 : _a.split('_')[0]) === null || _b === void 0 ? void 0 : _b.split('.')[0]) !== null && _c !== void 0 ? _c : 'en',
    debug: true,
    defaultNS: 'translation',
    keySeparator: false,
    backend: {
        loadPath: path.join(__dirname, '../locales/{{lng}}/{{ns}}.json'),
    },
});
exports.t = i18next_1.default.t.bind(i18next_1.default);
const logger = logger_1.getLogger({
    name: 'APIGW',
    middlewares: [logger_1.printToConsole],
});
/** 腾讯云 API 网关 for serverless component */
class Apigw {
    /**
     * 构造函数
     *
     * @param credentials 认证信息
     * @param region 地域信息
     */
    constructor(credentials, region = 'ap-guangzhou') {
        this.credentials = credentials;
        this.region = region;
        this.client = new apigateway_client_1.Client({
            credential: common_1.removeUndefined({
                secretId: credentials.SecretId,
                secretKey: credentials.SecretKey,
                token: credentials.Token,
            }),
            region: this.region,
            profile: {},
        });
        this.usagePlan = new usage_plan_1.UsagePlan(this);
        this.trigger = new trigger_1.Trigger(this);
        this.api = new api_1.Api(this);
        this.customDomain = new custom_domain_1.CustomDomain(this);
    }
    /**
     * 创建或更新 API 网关服务
     *
     * @param serviceConf API 网关创建或更新服务输入
     * @returns 创建成功后返回的信息
     */
    async createOrUpdateService(serviceConf) {
        const { environment, serviceId, protocols, netTypes, serviceName = 'Serverless_Framework', serviceDesc = 'Created By Serverless Framework', } = serviceConf;
        let serviceCreated = false;
        let exist = false;
        let detail;
        if (serviceId) {
            const res = await this.client.DescribeService({
                ServiceId: serviceId,
            });
            detail = res;
            if (detail) {
                exist = true;
                if (!(
                // FIXME: 小写？
                (serviceName === detail.ServiceName &&
                    serviceDesc === detail.ServiceDesc &&
                    protocols === detail.Protocol))) {
                    await this.client.ModifyService(common_1.pascalCaseProperty({
                        serviceId,
                        serviceDesc: serviceDesc || detail.ServiceDesc,
                        serviceName: serviceName || detail.ServiceName,
                        protocol: formatter_1.formatProtocolString(protocols),
                        netTypes: netTypes,
                    }));
                }
            }
        }
        if (!exist) {
            detail = await this.client.CreateService(common_1.pascalCaseProperty({
                serviceName: serviceName || 'Serverless_Framework',
                serviceDesc: serviceDesc || 'Created By Serverless Framework',
                protocol: formatter_1.formatProtocolString(protocols),
                netTypes,
            }));
            serviceCreated = true;
        }
        const outputs = {
            serviceName,
            serviceId: detail.ServiceId,
            subDomain: detail.OuterSubDomain && detail.InternalSubDomain
                ? [detail.OuterSubDomain, detail.InternalSubDomain]
                : detail.OuterSubDomain || detail.InternalSubDomain,
            serviceCreated,
            usagePlan: undefined,
        };
        if (serviceConf.usagePlan) {
            outputs.usagePlan = await this.usagePlan.bindUsagePlan({
                serviceId: detail.ServiceId,
                environment,
                usagePlanConfig: serviceConf.usagePlan,
                authConfig: serviceConf.auth,
            });
        }
        return common_1.deepClone(outputs);
    }
    /**
     * 部署 API 网关
     *
     * @param inputs 部署 API 网关输入
     * @returns 部署后的输出
     */
    async deploy(inputs) {
        const { environment = 'release', oldState = {} } = inputs;
        inputs.protocols = formatter_1.formatProtocolString(inputs.protocols);
        const { serviceId, serviceName, subDomain, serviceCreated, usagePlan, } = await this.createOrUpdateService(inputs);
        const apiList = [];
        const stateApiList = oldState.apiList || [];
        const endpoints = inputs.endpoints || [];
        const businessOauthApis = [];
        // deploy normal api
        for (let i = 0, len = endpoints.length; i < len; i++) {
            const endpoint = endpoints[i];
            if (endpoint.authType === 'OAUTH' && endpoint.businessType === 'NORMAL') {
                businessOauthApis.push(endpoint);
                continue;
            }
            const curApi = await this.api.apiDeployer({
                serviceId: serviceId,
                environment,
                apiList,
                oldList: stateApiList,
                apiConfig: endpoint,
            });
            apiList.push(curApi);
        }
        // deploy oauth business apis
        for (let i = 0, len = businessOauthApis.length; i < len; i++) {
            const endpoint = businessOauthApis[i];
            const curApi = await this.api.apiDeployer({
                serviceId: serviceId,
                environment,
                apiList,
                oldList: stateApiList,
                apiConfig: endpoint,
                isOauthApi: true,
            });
            apiList.push(curApi);
        }
        logger.info({
            msg: exports.t('正在发布服务 {{serviceId}}, 环境 {{environment}}', { serviceId, environment }),
        });
        await this.client.ReleaseService({
            ServiceId: serviceId,
            EnvironmentName: environment,
            ReleaseDesc: 'Released by Serverless Component',
        });
        logger.info({ msg: exports.t('发布服务 {{serviceId}} 成功', { serviceId }) });
        const outputs = {
            created: serviceCreated || oldState.created,
            serviceId,
            serviceName,
            subDomain,
            protocols: inputs.protocols,
            environment: environment,
            apiList,
        };
        // bind custom domain
        const customDomains = await this.customDomain.bindCustomDomain({
            serviceId: serviceId,
            subDomain: _.isArray(subDomain) ? subDomain[0] : subDomain,
            inputs,
        });
        if (customDomains.length > 0) {
            outputs.customDomains = customDomains;
        }
        if (usagePlan) {
            outputs.usagePlan = usagePlan;
        }
        return outputs;
    }
    /**
     * 移除 API 网关
     *
     * @param inputs 移除 API 网关参数
     */
    async remove(inputs) {
        const { created, environment, serviceId, apiList, customDomains, usagePlan } = inputs;
        // check service exist
        const detail = await this.client.DescribeService({
            ServiceId: serviceId,
        });
        if (!detail) {
            logger.info({ msg: exports.t('服务 {{serviceId}} 不存在', { serviceId }) });
            return;
        }
        // remove usage plan
        if (usagePlan) {
            await this.usagePlan.removeOrUnbindUsagePlan({
                serviceId,
                environment,
                usagePlan,
            });
        }
        // 1. remove all apis
        const oauthApis = [];
        for (let i = 0; i < apiList.length; i++) {
            const curApi = apiList[i];
            if (curApi.authType === 'OAUTH' && curApi.businessType === 'OAUTH') {
                oauthApis.push(curApi);
                continue;
            }
            await this.api.apiRemover({
                apiConfig: curApi,
                serviceId: serviceId,
                environment,
            });
        }
        for (let i = 0; i < oauthApis.length; i++) {
            const curApi = oauthApis[i];
            await this.api.apiRemover({
                apiConfig: curApi,
                serviceId: serviceId,
                environment,
            });
        }
        // 2. unbind all custom domains
        if (customDomains) {
            for (let i = 0; i < customDomains.length; i++) {
                const curDomain = customDomains[i];
                if (curDomain.subDomain && curDomain.created === true) {
                    logger.info({
                        msg: exports.t('解绑自定义域名 {{subDomain}}', { subDomain: curDomain.subDomain }),
                    });
                    await this.client.UnBindSubDomain({
                        ServiceId: serviceId,
                        SubDomain: curDomain.subDomain,
                    });
                }
            }
        }
        if (created === true) {
            // unrelease service
            logger.info({
                msg: exports.t('正在下线服务 {{serviceId}}, 环境 {{environment}}', { serviceId, environment }),
            });
            await try2be_1.try2Ignore(async () => {
                return this.client.UnReleaseService({
                    ServiceId: serviceId,
                    EnvironmentName: environment,
                });
            });
            logger.info({
                msg: exports.t('下线服务成功：{{serviceId}}, {{environment}}', { serviceId, environment }),
            });
            // delete service
            logger.info({ msg: exports.t('正在删除服务 {{serviceId}}', { serviceId }) });
            await try2be_1.try2Ignore(async () => {
                const r = await this.client.DeleteService({
                    ServiceId: serviceId,
                });
                return r;
            });
            logger.info({ msg: exports.t('删除服务 {{serviceId}} 成功', { serviceId }) });
        }
    }
}
exports.Apigw = Apigw;
