"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Trigger = void 0;
const logger_1 = require("@jsonlog/logger");
const common_1 = require("@tencent-serverless-framework/common");
const scf_client_1 = require("tencentcloud-sdk-nodejs/tencentcloud/services/scf/v20180416/scf_client");
const _1 = require(".");
const logger = logger_1.getLogger({
    name: 'APIGW',
    middlewares: [logger_1.printToConsole],
});
/** API 网关组件触发器子模块 */
class Trigger {
    /**
     * 构造函数
     *
     * @param apigw API 网关组件（依赖注入）
     */
    constructor(apigw) {
        this.apigw = apigw;
        this.scfClient = new scf_client_1.Client({
            credential: common_1.removeUndefined({
                secretId: this.apigw.credentials.SecretId,
                secretKey: this.apigw.credentials.SecretKey,
                token: this.apigw.credentials.Token,
            }),
            region: this.apigw.region,
            profile: {},
        });
    }
    /**
     * 根据 API 网关 API 和函数信息删除触发器
     *
     * @param inputs 输入参数
     */
    async removeScfTrigger(inputs) {
        const { serviceId, apiId, functionName, namespace, qualifier } = inputs;
        // 1. get all trigger list
        const res = await this.scfClient.ListTriggers({
            FunctionName: functionName,
            Namespace: namespace,
        });
        const allList = res.Triggers;
        // 2. get apigw trigger list
        const apigwList = allList.filter((item) => item.Type === 'apigw');
        const [curApiTrigger] = apigwList.filter((trigger) => {
            return trigger.ResourceId.indexOf(`service/${serviceId}/API/${apiId}`) !== -1;
        });
        // 3. remove current apigw trigger
        if (curApiTrigger) {
            try {
                await this.scfClient.DeleteTrigger({
                    Type: 'apigw',
                    FunctionName: functionName,
                    Namespace: namespace,
                    Qualifier: qualifier,
                    TriggerDesc: curApiTrigger.TriggerDesc,
                    TriggerName: curApiTrigger.TriggerName,
                });
            }
            catch (e) {
                logger.info({ msg: _1.t('发生错误：{{e}}', { e }) });
            }
        }
    }
    /**
     * 删除给定 API 触发器
     *
     * @param inputs 输入
     * @returns 是否删除成功
     */
    async removeTrigger(inputs) {
        const { serviceId, apiId } = inputs;
        // get api detail
        const res = await this.apigw.client.DescribeApi({
            ServiceId: serviceId,
            ApiId: apiId,
        });
        const apiDetail = res.Result;
        if (!apiDetail) {
            return true;
        }
        // 1. scf type
        if (apiDetail.ServiceScfFunctionName) {
            await this.removeScfTrigger({
                serviceId,
                apiId,
                functionName: apiDetail.ServiceScfFunctionName,
                namespace: apiDetail.ServiceScfFunctionNamespace,
                qualifier: apiDetail.ServiceScfFunctionQualifier,
            });
        }
        // 2. ws type
        if (apiDetail.ServiceWebsocketRegisterFunctionName) {
            await this.removeScfTrigger({
                serviceId,
                apiId,
                functionName: apiDetail.ServiceWebsocketRegisterFunctionName,
                namespace: apiDetail.ServiceWebsocketRegisterFunctionNamespace,
                qualifier: apiDetail.ServiceWebsocketRegisterFunctionQualifier,
            });
        }
        if (apiDetail.ServiceWebsocketCleanupFunctionName) {
            await this.removeScfTrigger({
                serviceId,
                apiId,
                functionName: apiDetail.ServiceWebsocketCleanupFunctionName,
                namespace: apiDetail.ServiceWebsocketCleanupFunctionNamespace,
                qualifier: apiDetail.ServiceWebsocketCleanupFunctionQualifier,
            });
        }
        if (apiDetail.ServiceWebsocketTransportFunctionName) {
            await this.removeScfTrigger({
                serviceId,
                apiId,
                functionName: apiDetail.ServiceWebsocketTransportFunctionName,
                namespace: apiDetail.ServiceWebsocketTransportFunctionNamespace,
                qualifier: apiDetail.ServiceWebsocketTransportFunctionQualifier,
            });
        }
        return true;
    }
}
exports.Trigger = Trigger;
