"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UsagePlan = exports.uniqueArray = void 0;
const logger_1 = require("@jsonlog/logger");
const common_1 = require("@tencent-serverless-framework/common");
const try2be_1 = require("try2be");
const _1 = require(".");
const logger = logger_1.getLogger({
    name: 'APIGW',
    middlewares: [logger_1.printToConsole],
});
/**
 * 去除数组的重复值
 *
 * @param arr 数组
 * @returns 去除重复值后的数组
 */
function uniqueArray(arr) {
    return arr.filter((item, index, self) => {
        return self.indexOf(item) === index;
    });
}
exports.uniqueArray = uniqueArray;
/** API 网关组件用量计划域名子模块 */
class UsagePlan {
    /**
     * 构造函数
     *
     * @param apigw API 网关组件（依赖注入）
     */
    constructor(apigw) {
        this.apigw = apigw;
    }
    /**
     * 设置一对 API 密钥
     *
     * @param inputs 输入
     * @returns 密钥 id 列表
     */
    async setupUsagePlanSecret(inputs) {
        const { secretName, secretIds, created } = inputs;
        const secretIdsOutput = {
            created: !!created,
            secretIds,
        };
        // user not setup secret ids, just auto generate one
        if (!secretIds || secretIds.length === 0) {
            logger.info({ msg: _1.t('创建一个新的密钥') });
            const res = await this.apigw.client.CreateApiKey({
                SecretName: secretName,
                AccessKeyType: 'auto',
            });
            const { AccessKeyId, AccessKeySecret } = res.Result;
            logger.info({
                msg: _1.t('密钥 ID {{AccessKeyId}} 和 Key {{AccessKeySecret}} 创建成功', {
                    AccessKeyId,
                    AccessKeySecret,
                }),
            });
            secretIdsOutput.secretIds = [AccessKeyId];
            secretIdsOutput.created = true;
        }
        else {
            // use setup secret ids
            // 1. unique it
            // 2. make sure all bind secret ids exist in user's list
            const uniqSecretIds = uniqueArray(secretIds);
            // get all secretId, check local secretId exists
            const res = await this.apigw.client.DescribeApiKeysStatus({
                Limit: uniqSecretIds.length,
                Filters: [
                    {
                        Name: 'AccessKeyId',
                        Values: uniqSecretIds,
                    },
                ],
            });
            const { ApiKeySet } = res.Result;
            const existKeysLen = ApiKeySet.length;
            // Filter invalid and non-existent keys
            const ids = [];
            uniqSecretIds.forEach((secretId) => {
                let found = false;
                let disable = false;
                for (let n = 0; n < existKeysLen; n++) {
                    if (ApiKeySet[n] && secretId === ApiKeySet[n].AccessKeyId) {
                        if (Number(ApiKeySet[n].Status) === 1) {
                            found = true;
                        }
                        else {
                            disable = true;
                            logger.info({ msg: _1.t('有一个被禁用的 {{secretId}} 无法被找到', { secretId }) });
                        }
                        break;
                    }
                }
                if (!found) {
                    if (!disable) {
                        logger.info({ msg: _1.t('密钥 ID {{secretId}} 不存在', { secretId }) });
                    }
                }
                else {
                    ids.push(secretId);
                }
            });
            secretIdsOutput.secretIds = ids;
        }
        return secretIdsOutput;
    }
    /**
     * 获取 secrets 列表
     *
     * @param usagePlanId 用户计划 id
     * @param secrets 使用计划绑定密钥数组
     * @param options 获取时的选项
     * @param options.limit 至多获取多少项
     * @param options.offset 跳过多少项开始获取
     * @returns 使用计划 secrets 列表
     */
    async getAllBoundSecrets(usagePlanId, secrets = [], options) {
        const { limit, offset = 0 } = options;
        const res = await this.apigw.client.DescribeUsagePlanSecretIds({
            UsagePlanId: usagePlanId,
            Limit: limit,
            Offset: offset,
        });
        const { AccessKeyList } = res.Result;
        if (AccessKeyList.length < limit) {
            return AccessKeyList;
        }
        const more = await this.getAllBoundSecrets(usagePlanId, AccessKeyList, {
            limit,
            offset: offset + AccessKeyList.length,
        });
        // FIXME: more is same type with res, why concat?
        // return res.concat(more.AccessKeyList);
        return secrets.concat(more);
    }
    /**
     * 找到所有未绑定的使用计划 secretIds
     *
     * @param root0 输入参数
     * @param root0.usagePlanId 使用计划 ID
     * @param root0.secretIds 使用计划密钥 ID 列表
     * @returns 找到的不存在的使用计划 id 列表
     */
    async getUnboundSecretIds({ usagePlanId, secretIds, }) {
        const allBoundSecretObjs = await this.getAllBoundSecrets(usagePlanId, [], { limit: 100 });
        const allBoundSecretIds = allBoundSecretObjs.map((item) => item.AccessKeyId);
        const unboundSecretIds = secretIds.filter((item) => {
            if (allBoundSecretIds.indexOf(item) === -1) {
                return true;
            }
            logger.info({
                msg: _1.t('用户计划 {{usagePlanId}} 和密钥 ID {{item}} 已经被绑定', { usagePlanId, item }),
            });
            return false;
        });
        return unboundSecretIds;
    }
    /**
     * 设置 API 网关的使用计划
     *
     * @param inputs API 网关的使用计划输入参数
     * @returns 使用计划的 id
     **/
    async setupUsagePlan(inputs) {
        var _a, _b, _c, _d;
        const { usagePlan } = inputs;
        const usageInputs = {
            usagePlanName: (_a = usagePlan.usagePlanName) !== null && _a !== void 0 ? _a : '',
            usagePlanDesc: (_b = usagePlan.usagePlanDesc) !== null && _b !== void 0 ? _b : '',
            maxRequestNumPreSec: (_c = usagePlan.maxRequestNumPreSec) !== null && _c !== void 0 ? _c : -1,
            maxRequestNum: (_d = usagePlan.maxRequestNum) !== null && _d !== void 0 ? _d : -1,
        };
        const usagePlanOutput = {
            created: usagePlan.created || false,
            usagePlanId: usagePlan.usagePlanId,
        };
        let exist = false;
        if (usagePlan.usagePlanId) {
            try {
                const res = await this.apigw.client.DescribeUsagePlan({
                    UsagePlanId: usagePlan.usagePlanId,
                });
                const detail = res.Result;
                if (detail && detail.UsagePlanId) {
                    exist = true;
                }
            }
            catch (e) {
                // no op
            }
        }
        if (exist) {
            logger.info({
                msg: _1.t('正在更新用量计划 {{usagePlanId}}', { usagePlanId: usagePlan.usagePlanId }),
            });
            await this.apigw.client.ModifyUsagePlan(common_1.removeUndefined({
                UsagePlanId: usagePlanOutput.usagePlanId,
                UsagePlanName: usageInputs.usagePlanName,
                UsagePlanDesc: usageInputs.usagePlanDesc,
                MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
                MaxRequestNum: usageInputs.maxRequestNum,
            }));
        }
        else {
            const res = await this.apigw.client.CreateUsagePlan(common_1.removeUndefined({
                UsagePlanName: usageInputs.usagePlanName,
                UsagePlanDesc: usageInputs.usagePlanDesc,
                MaxRequestNumPreSec: usageInputs.maxRequestNumPreSec,
                MaxRequestNum: usageInputs.maxRequestNum,
            }));
            const { UsagePlanId } = res.Result;
            usagePlanOutput.usagePlanId = UsagePlanId;
            usagePlanOutput.created = true;
            logger.info({
                msg: _1.t('用量计划 {{usagePlanId}} 已经创建', { usagePlanId: usagePlanOutput.usagePlanId }),
            });
        }
        return usagePlanOutput;
    }
    /**
     * API 网关绑定用量计划
     *
     * @param inputs 输入
     * @returns 输出，绑定后的用量计划
     */
    async bindUsagePlan(inputs) {
        const { apiId, serviceId, environment, usagePlanConfig, authConfig } = inputs;
        const usagePlan = await this.setupUsagePlan({
            usagePlan: usagePlanConfig,
        });
        if (authConfig) {
            const { secretIds = [] } = authConfig;
            const secrets = await this.setupUsagePlanSecret({
                secretName: authConfig.secretName,
                secretIds,
            });
            const unboundSecretIds = await this.getUnboundSecretIds({
                usagePlanId: usagePlan.usagePlanId,
                secretIds: secrets.secretIds,
            });
            if (unboundSecretIds.length > 0) {
                logger.info({
                    msg: _1.t('正在绑定密钥 {{unboundSecretIds}} 到用量计划 {{usagePlanId}}', {
                        unboundSecretIds,
                        usagePlanId: usagePlan.usagePlanId,
                    }),
                });
                await this.apigw.client.BindSecretIds({
                    UsagePlanId: usagePlan.usagePlanId,
                    AccessKeyIds: unboundSecretIds,
                });
                logger.info({ msg: _1.t('绑定密钥成功') });
            }
            // store in api list
            usagePlan.secrets = secrets;
        }
        const res = await this.apigw.client.DescribeApiUsagePlan({
            ServiceId: serviceId,
            Limit: 100,
        });
        const { ApiUsagePlanList } = res.Result;
        const oldUsagePlan = ApiUsagePlanList.find((item) => {
            return apiId
                ? item.UsagePlanId === usagePlan.usagePlanId && item.ApiId === apiId
                : item.UsagePlanId === usagePlan.usagePlanId;
        });
        if (oldUsagePlan) {
            if (apiId) {
                logger.info({
                    msg: _1.t('用量计划 {{usagePlanId}} 已经被绑定到 API {{apiId}}', {
                        usagePlan: usagePlan.usagePlanId,
                        apiId,
                    }),
                });
            }
            else {
                logger.info({
                    msg: _1.t('用量计划 {{usagePlanId}} 已经被绑定到环境 {{environment}}', {
                        usagePlanId: usagePlan.usagePlanId,
                        environment,
                    }),
                });
            }
            return usagePlan;
        }
        if (apiId) {
            logger.info({
                msg: _1.t('正在绑定用量计划 {{usagePlanId}} 到 API {{apiId}}', {
                    usagePlanId: usagePlan.usagePlanId,
                    apiId,
                }),
            });
            await this.apigw.client.BindEnvironment({
                ServiceId: serviceId,
                Environment: environment,
                BindType: 'API',
                UsagePlanIds: [usagePlan.usagePlanId],
                ApiIds: [apiId],
            });
            logger.info({
                msg: _1.t('绑定用量计划 {{usagePlan.usagePlanId}} 到 API {{apiId}} 成功', {
                    usagePlan: usagePlan.usagePlanId,
                    apiId,
                }),
            });
            return usagePlan;
        }
        logger.info({
            msg: _1.t('正在绑定用量计划 {{usagePlanId}} 到环境 {{environment}}', {
                usagePlanId: usagePlan.usagePlanId,
                environment,
            }),
        });
        await this.apigw.client.BindEnvironment({
            ServiceId: serviceId,
            Environment: environment,
            BindType: 'SERVICE',
            UsagePlanIds: [usagePlan.usagePlanId],
        });
        logger.info({
            msg: _1.t('绑定用量计划 {{usagePlanId}} 到环境 {{environment}} 成功', {
                usagePlanId: usagePlan.usagePlanId,
                environment,
            }),
        });
        return usagePlan;
    }
    /**
     * 移除 API 网关的使用计划
     *
     * @param inputs 移除 API 网关的使用计划输入参数
     */
    async removeOrUnbindUsagePlan(inputs) {
        var _a;
        const { serviceId, environment, usagePlan, apiId } = inputs;
        // 1.1 unbind secrete ids
        const { secrets } = usagePlan;
        if (secrets && secrets.secretIds) {
            await try2be_1.try2Ignore(async () => {
                return this.apigw.client.UnBindSecretIds({
                    AccessKeyIds: secrets.secretIds,
                    UsagePlanId: usagePlan.usagePlanId,
                });
            });
            logger.info({
                msg: _1.t('正在从用量计划 {{usagePlanId}} 解绑密钥', { usagePlanId: usagePlan.usagePlanId }),
            });
            // delelet all created api key
            if (((_a = usagePlan.secrets) === null || _a === void 0 ? void 0 : _a.created) === true) {
                for (let sIdx = 0; sIdx < secrets.secretIds.length; sIdx++) {
                    const secretId = secrets.secretIds[sIdx];
                    logger.info({ msg: _1.t('正在移除密钥 {{secretId}}}', { secretId }) });
                    await try2be_1.try2Ignore(async () => {
                        return this.apigw.client.DisableApiKey({
                            AccessKeyId: secretId,
                        });
                    });
                    await try2be_1.try2Ignore(async () => {
                        return this.apigw.client.DeleteApiKey({
                            AccessKeyId: secretId,
                        });
                    });
                }
            }
        }
        // 1.2 unbind environment
        if (apiId) {
            await try2be_1.try2Ignore(async () => {
                return this.apigw.client.UnBindEnvironment({
                    ServiceId: serviceId,
                    UsagePlanIds: [usagePlan.usagePlanId],
                    Environment: environment,
                    BindType: 'API',
                    ApiIds: [apiId],
                });
            });
        }
        else {
            await try2be_1.try2Ignore(async () => {
                return this.apigw.client.UnBindEnvironment({
                    ServiceId: serviceId,
                    UsagePlanIds: [usagePlan.usagePlanId],
                    Environment: environment,
                    BindType: 'SERVICE',
                });
            });
        }
        logger.info({
            msg: _1.t('正在从服务 {{serviceId}} 解绑用量计划 {{usagePlanId}}', {
                serviceId,
                usagePlanId: usagePlan.usagePlanId,
            }),
        });
        // 1.3 delete created usage plan
        if (usagePlan.created === true) {
            logger.info({
                msg: _1.t('正在移除用量计划 {{usagePlanId}}', { usagePlanId: usagePlan.usagePlanId }),
            });
            await try2be_1.try2Ignore(async () => {
                return this.apigw.client.DeleteUsagePlan({
                    UsagePlanId: usagePlan.usagePlanId,
                });
            });
        }
    }
}
exports.UsagePlan = UsagePlan;
