import {
  ApiCredentials,
  deepClone,
  pascalCaseProperty,
  RegionType,
  removeUndefined,
} from '@tencent-serverless-framework/common';
import { try2Ignore } from 'try2be';
import {
  ApigwCreateOrUpdateServiceInputs,
  ApiDeployerOutputs,
  ApigwDeployInputs,
  ApigwEndpoint,
  ApigwDeployOutputs,
  ApigwRemoveInputs,
  ApigwUsagePlan,
} from './interface';
import * as _ from 'lodash';
import { Client } from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_client';
import { DescribeServiceResponse } from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';
import { UsagePlan } from './usage-plan';
import { Trigger } from './trigger';
import { Api } from './api';
import { formatProtocolString } from './formatter';
import { CustomDomain } from './custom-domain';
import { getLogger, printToConsole } from '@jsonlog/logger';
import i18next from 'i18next';
import Backend from 'i18next-fs-backend';
import * as path from 'path';

i18next.use(Backend).init({
  initImmediate: false,
  fallbackLng: 'zh',
  lng: process.env.LANG?.split('_')[0]?.split('.')[0] ?? 'en',
  debug: true,
  defaultNS: 'translation',
  keySeparator: false,
  backend: {
    loadPath: path.join(__dirname, '../locales/{{lng}}/{{ns}}.json'),
  },
});

export const t = i18next.t.bind(i18next);

const logger = getLogger({
  name: 'APIGW',
  middlewares: [printToConsole],
});

/** 腾讯云 API 网关 for serverless component */
export class Apigw {
  /** 认证信息 */
  credentials: ApiCredentials;
  /** API 网关客户端 */
  client: Client;
  /** 区域信息 */
  region: RegionType;

  /** 用量计划子模块 */
  usagePlan: UsagePlan;
  /** 触发器子模块 */
  trigger: Trigger;
  /** API (Endpoint )子模块 */
  api: Api;
  /** 自定义域名子模块 */
  customDomain: CustomDomain;

  /**
   * 构造函数
   *
   * @param credentials 认证信息
   * @param region 地域信息
   */
  constructor(credentials: ApiCredentials, region: RegionType = 'ap-guangzhou') {
    this.credentials = credentials;
    this.region = region;
    this.client = new Client({
      credential: removeUndefined({
        secretId: credentials.SecretId,
        secretKey: credentials.SecretKey,
        token: credentials.Token,
      }),
      region: this.region,
      profile: {},
    });

    this.usagePlan = new UsagePlan(this);
    this.trigger = new Trigger(this);
    this.api = new Api(this);
    this.customDomain = new CustomDomain(this);
  }

  /**
   * 创建或更新 API 网关服务
   *
   * @param serviceConf API 网关创建或更新服务输入
   * @returns 创建成功后返回的信息
   */
  async createOrUpdateService(serviceConf: ApigwCreateOrUpdateServiceInputs) {
    const {
      environment,
      serviceId,
      protocols,
      netTypes,
      serviceName = 'Serverless_Framework',
      serviceDesc = 'Created By Serverless Framework',
    } = serviceConf;
    let serviceCreated = false;
    let exist = false;

    let detail: DescribeServiceResponse;
    if (serviceId) {
      const res = await this.client.DescribeService({
        ServiceId: serviceId,
      });
      detail = res;
      if (detail) {
        exist = true;
        if (
          !(
            // FIXME: 小写？
            (
              serviceName === detail.ServiceName &&
              serviceDesc === detail.ServiceDesc &&
              protocols === detail.Protocol
            )
          )
        ) {
          await this.client.ModifyService(
            pascalCaseProperty({
              serviceId,
              serviceDesc: serviceDesc || detail.ServiceDesc,
              serviceName: serviceName || detail.ServiceName,
              protocol: formatProtocolString(protocols),
              netTypes: netTypes,
            }),
          );
        }
      }
    }

    if (!exist) {
      detail = await this.client.CreateService(
        pascalCaseProperty({
          serviceName: serviceName || 'Serverless_Framework',
          serviceDesc: serviceDesc || 'Created By Serverless Framework',
          protocol: formatProtocolString(protocols),
          netTypes,
        }),
      );
      serviceCreated = true;
    }

    const outputs = {
      serviceName,
      serviceId: detail!.ServiceId,
      subDomain:
        detail!.OuterSubDomain && detail!.InternalSubDomain
          ? [detail!.OuterSubDomain, detail!.InternalSubDomain]
          : detail!.OuterSubDomain || detail!.InternalSubDomain,
      serviceCreated,
      usagePlan: undefined as undefined | ApigwUsagePlan,
    };

    if (serviceConf.usagePlan) {
      outputs.usagePlan = await this.usagePlan.bindUsagePlan({
        serviceId: detail!.ServiceId,
        environment,
        usagePlanConfig: serviceConf.usagePlan,
        authConfig: serviceConf.auth,
      });
    }

    return deepClone(outputs);
  }

  /**
   * 部署 API 网关
   *
   * @param inputs 部署 API 网关输入
   * @returns 部署后的输出
   */
  async deploy(inputs: ApigwDeployInputs) {
    const { environment = 'release' as const, oldState = {} } = inputs;
    inputs.protocols = formatProtocolString(inputs.protocols as ('http' | 'https')[]);

    const {
      serviceId,
      serviceName,
      subDomain,
      serviceCreated,
      usagePlan,
    } = await this.createOrUpdateService(inputs)!;

    const apiList: ApigwEndpoint[] = [];
    const stateApiList = oldState.apiList || [];

    const endpoints = inputs.endpoints || [];

    const businessOauthApis = [];
    // deploy normal api
    for (let i = 0, len = endpoints.length; i < len; i++) {
      const endpoint = endpoints[i];
      if (endpoint.authType === 'OAUTH' && endpoint.businessType === 'NORMAL') {
        businessOauthApis.push(endpoint);
        continue;
      }
      const curApi: ApiDeployerOutputs = await this.api.apiDeployer({
        serviceId: serviceId!,
        environment,
        apiList,
        oldList: stateApiList,
        apiConfig: endpoint,
      });
      apiList.push(curApi);
    }

    // deploy oauth business apis
    for (let i = 0, len = businessOauthApis.length; i < len; i++) {
      const endpoint = businessOauthApis[i];
      const curApi = await this.api.apiDeployer({
        serviceId: serviceId!,
        environment,
        apiList,
        oldList: stateApiList,
        apiConfig: endpoint,
        isOauthApi: true,
      });
      apiList.push(curApi);
    }

    logger.info({
      msg: t('正在发布服务 {{serviceId}}, 环境 {{environment}}', { serviceId, environment }),
    });
    await this.client.ReleaseService({
      ServiceId: serviceId!,
      EnvironmentName: environment,
      ReleaseDesc: 'Released by Serverless Component',
    });
    logger.info({ msg: t('发布服务 {{serviceId}} 成功', { serviceId }) });

    const outputs: ApigwDeployOutputs = {
      created: serviceCreated || oldState.created,
      serviceId,
      serviceName,
      subDomain,
      protocols: inputs.protocols,
      environment: environment,
      apiList,
    };

    // bind custom domain
    const customDomains = await this.customDomain.bindCustomDomain({
      serviceId: serviceId!,
      subDomain: _.isArray(subDomain) ? subDomain[0] : subDomain!,
      inputs,
    });
    if (customDomains.length > 0) {
      outputs.customDomains = customDomains;
    }

    if (usagePlan) {
      outputs.usagePlan = usagePlan;
    }

    return outputs;
  }

  /**
   * 移除 API 网关
   *
   * @param inputs 移除 API 网关参数
   */
  async remove(inputs: ApigwRemoveInputs) {
    const { created, environment, serviceId, apiList, customDomains, usagePlan } = inputs;

    // check service exist
    const detail = await this.client.DescribeService({
      ServiceId: serviceId!,
    });

    if (!detail) {
      logger.info({ msg: t('服务 {{serviceId}} 不存在', { serviceId }) });
      return;
    }

    // remove usage plan
    if (usagePlan) {
      await this.usagePlan.removeOrUnbindUsagePlan({
        serviceId,
        environment,
        usagePlan,
      });
    }

    // 1. remove all apis
    const oauthApis = [];
    for (let i = 0; i < apiList.length; i++) {
      const curApi = apiList[i];
      if (curApi.authType === 'OAUTH' && curApi.businessType === 'OAUTH') {
        oauthApis.push(curApi);
        continue;
      }

      await this.api.apiRemover({
        apiConfig: curApi,
        serviceId: serviceId!,
        environment,
      });
    }
    for (let i = 0; i < oauthApis.length; i++) {
      const curApi = oauthApis[i];
      await this.api.apiRemover({
        apiConfig: curApi,
        serviceId: serviceId!,
        environment,
      });
    }

    // 2. unbind all custom domains
    if (customDomains) {
      for (let i = 0; i < customDomains.length; i++) {
        const curDomain = customDomains[i];
        if (curDomain.subDomain && curDomain.created === true) {
          logger.info({
            msg: t('解绑自定义域名 {{subDomain}}', { subDomain: curDomain.subDomain }),
          });
          await this.client.UnBindSubDomain({
            ServiceId: serviceId!,
            SubDomain: curDomain.subDomain,
          });
        }
      }
    }

    if (created === true) {
      // unrelease service
      logger.info({
        msg: t('正在下线服务 {{serviceId}}, 环境 {{environment}}', { serviceId, environment }),
      });
      await try2Ignore(async () => {
        return this.client.UnReleaseService({
          ServiceId: serviceId!,
          EnvironmentName: environment,
        });
      });

      logger.info({
        msg: t('下线服务成功：{{serviceId}}, {{environment}}', { serviceId, environment }),
      });

      // delete service
      logger.info({ msg: t('正在删除服务 {{serviceId}}', { serviceId }) });
      await try2Ignore(async () => {
        const r = await this.client.DeleteService({
          ServiceId: serviceId!,
        });
        return r;
      });

      logger.info({ msg: t('删除服务 {{serviceId}} 成功', { serviceId }) });
    }
  }
}
