import { CamelCasedProps, RegionType } from '@tencent-serverless-framework/common';
import {
  Base64EncodedTriggerRule,
  OauthConfig,
} from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';

/** API 网关发布环境类型 */
export type ApigwEnviromentType = 'release' | 'prepub' | 'test';

/** API 网关用量计划 */
export interface ApigwUsagePlan {
  /** API 网关用量计划 */
  usagePlanId: string;
  /**
   * 用户自定义的使用计划名称。
   */
  usagePlanName?: string;
  /**
   * 用户自定义的使用计划描述。
   */
  usagePlanDesc?: string;
  /**
   * 请求配额总数，取值范围为-1或者[1, 99999999]，默认为-1，表示不开启。
   */
  maxRequestNumPreSec?: number;
  /**
   * 每秒请求限制数，取值范围为-1或者[1, 2000]，默认-1，表示不开启。
   */
  maxRequestNum?: number;

  /** 是否被创建，记录用 */
  created?: boolean;

  /**
   * 密钥 ID
   */
  secrets?: {
    /**
     * 密钥 ID 数组。
     */
    secretIds?: string[];
    /** 是否被创建，记录用 */
    created: boolean;
  };
}

/** API 网关用量计划输入 */
export interface ApigwSetupUsagePlanInputs {
  /** API 网关用量计划 */
  usagePlan: ApigwUsagePlan;
}

/** API 网关设置用量计划密钥输入 */
export interface ApigwSetupUsagePlanSecretInputs {
  /** 要使用的密钥 id 列表 */
  secretIds?: string[];
  /** 用户自定义的密钥名 */
  secretName: string;

  /** 是否成功创建 */
  created?: boolean;
}

/** API 网关绑定用户计划输入 */
export interface ApigwBindUsagePlanInputs {
  /**
   * API 接口唯一 ID。
   */
  apiId?: string;
  /**
   * 待修改服务的唯一 Id。
   */
  serviceId?: string;
  /**
   * 待发布的环境名称，当前支持三个环境，test（测试环境）、prepub（预发布环境）和 release（发布环境）。
   */
  environment?: ApigwEnviromentType;
  /** API 网关用量计划配置 */
  usagePlanConfig: ApigwUsagePlan;
  /** API 网关设置用量计划密钥输入配置 */
  authConfig?: ApigwSetupUsagePlanSecretInputs;
}

/** API 网关绑定用户计划输出 */
export interface ApigwBindUsagePlanOutputs extends ApigwBindUsagePlanInputs {}

/** API 网关 API */
export interface ApigwEndpoint {
  /** 记录是否创建 */
  created?: boolean;
  /**
   * API 接口唯一 ID。
   */
  apiId?: string;
  /** API 网关用量计划 */
  usagePlan?: ApigwUsagePlan;
  /** API 网关设置用量计划密钥输入 */
  auth?: ApigwSetupUsagePlanSecretInputs;
  /**
   * API 鉴权类型。取值为SECRET（密钥对鉴权）、NONE（免鉴权）、OAUTH。
   */
  authType?: 'NONE' | string;
  /**
   * OAUTH API的类型。当AuthType 为 OAUTH时该字段有效， 取值为NORMAL（业务API）和 OAUTH（授权API）。
   */
  businessType?: 'NORMAL' | string;
  /**
   * API 的后端服务路径，如 /path
   */
  path: string;
  /**
   * API的后端服务请求方法，如 GET
   */
  method: string;
  /**
   * 用户自定义的 API 名称。
   */
  apiName?: string;
  /**
   * API 的前端请求类型，如 HTTP 或 HTTPS 或者 HTTP 和 HTTPS。
   */
  protocol?: 'HTTP' | 'HTTPS' | 'WEBSOCKET';
  /** 描述 */
  description?: string;
  /**
   * API 的后端服务类型。可取值为 HTTP、MOCK、TSF、CLB、SCF、WEBSOCKET、TARGET（内测）。
   */
  serviceType?: 'SCF' | 'MOCK' | 'TSF' | 'CLB' | 'HTTP' | 'MOCK' | 'WEBSOCKET' | 'TARGT';
  /**
   * API 的后端服务超时时间，单位是秒。
   */
  serviceTimeout?: 15;
  /**
   * 返回类型。
   */
  responseType?: 'HTML' | string;
  /**
   * 是否开启跨域。
   */
  enableCORS?: boolean;
  /**
   * Oauth授权配置信息
   */
  oauthConfig?: CamelCasedProps<OauthConfig>;
  /**
   * 关联的授权API 唯一 ID，当AuthType为OAUTH且ApiBusinessType为NORMAL时生效。标示业务API绑定的oauth2.0授权API唯一ID。
   */
  authRelationApiId?: string;
  /**
   * 授权 API 方法和路径
   */
  authRelationApi?: {
    /**
     * API 的请求方法，如 GET。
     */
    method: string;
    /**
     * API 的路径，如 /path。
     */
    path: string;
  };
  /**
   * WEBSOCKET 回推地址。
   */
  internalDomain?: string;
  /**
   * 是否打开Base64编码，只有后端是scf时才会生效。
   */
  isBase64Encoded?: boolean;
  /**
   * 是否开启Base64编码的header触发，只有后端为scf时才会生效。
   */
  isBase64Trigger?: boolean;
  /**
   * Base64编码的header触发规则
   */
  base64EncodedTriggerRules?: CamelCasedProps<Base64EncodedTriggerRule>[];

  /** API serviceType=SCF 目标函数 */
  function?: {
    /** 函数名称 */
    functionName?: string;
    /** 函数命名空间 */
    functionNamespace?: string;
    /** 函数版本 */
    functionQualifier?: string;
    /**  */
    transportFunctionName?: string;
    /**  */
    registerFunctionName?: string;
  };

  /** API serviceType=MOCK 返回信息 */
  serviceMockReturnMessage?: string;

  /** API serviceType=HTTP 服务配置 */
  serviceConfig?: {
    /** 源 url */
    url: string;
    /** 路径 */
    path: string;
    /** 方法 */
    method: string;
  };
}

/** API 网关自定义域名 */
export interface ApigwCustomDomain {
  /**
   * 待绑定的自定义的域名。
   */
  domain: string;
  /**
   * 服务支持协议，可选值为http、https、http&https。
   */
  protocols: ('http' | 'https')[];
  /**
   * 待绑定自定义域名的证书唯一 ID。针对Protocol 为https或http&https可以选择上传。
   */
  certificateId?: string;
  /**
   * 是否修改为使用默认路径映射。为 true，表示使用默认路径映射，为 false，表示使用自定义路径映射。
   */
  isDefaultMapping?: boolean;
  /**
   * 修改后的路径映射列表。
   */
  pathMappingSet?: {
    /** 路径 */
    path: string;
    /** 映射到的环境 */
    environment: string;
  }[];
  /**
   * 网络类型，可选值为OUTER、INNER。
   */
  netType?: string;
  /**
   * 是否将HTTP请求强制跳转 HTTPS，默认为false。参数为 true时，API网关会将所有使用该自定义域名的 HTTP 协议的请求重定向至 HTTPS 协议进行转发。
   */
  isForcedHttps?: boolean;
  /**
   * 待绑定的自定义的域名。
   */
  subDomain?: string;
  /** 服务是否创建 */
  created?: boolean;
}

/** API 网关自定义域名输入 */
export interface ApigwBindCustomDomainInputs {
  /** 要配置的自定义域名列表 */
  customDomains?: ApigwCustomDomain[];
  /** 要配置的自定义域名列表协议 */
  protocols: ('http' | 'https')[] | string;
  /** 旧的自定义域名状态，用于判断是否需要解绑自定义域名 */
  oldState?: Partial<ApigwBindCustomDomainInputs>;
}

/** Api 网关创建或更新输入 */
export interface ApigwCreateOrUpdateServiceInputs {
  /**
   * 待切换的环境名称，当前支持三个环境，test（测试环境）、prepub（预发布环境）和 release（发布环境）。
   */
  environment?: ApigwEnviromentType;
  /**
   * API 的前端请求类型，如 HTTP 或 HTTPS 或者 HTTP 和 HTTPS。
   */
  protocols: ('http' | 'https')[] | string;
  /**
   * 网络类型列表，用于指定支持的访问类型，INNER为内网访问，OUTER为外网访问。默认为OUTER。
   */
  netTypes?: string[];
  /**
   * 用户自定义的服务名称。
   */
  serviceName?: string;
  /**
   * 用户自定义的服务描述。
   */
  serviceDesc?: string;
  /**
   * 待切换服务的唯一 Id。
   */
  serviceId?: string;

  /** API 网关用量计划 */
  usagePlan?: ApigwUsagePlan;
  /** API 网关设置用量计划密钥输入 */
  auth?: ApigwSetupUsagePlanSecretInputs;
}

/** 部署 API 网关 API 输入 */
export type ApiDeployerOutputs = ApigwEndpoint;

/** 创建或更新 API 网关 API 输入 */
export interface CreateOrUpdateApiInputs {
  /**
   * 服务唯一ID。
   */
  serviceId: string;
  /** API 网关 API */
  endpoint: ApigwEndpoint;
  /**
   * 环境名称，当前支持三个环境，test（测试环境）、prepub（预发布环境）和 release（发布环境）。
   */
  environment: ApigwEnviromentType;
  /** 是否已经创建 */
  created?: boolean;
}

/** 创建 API 网关 API 输入 */
export interface ApiDeployerInputs {
  /**
   * 服务唯一ID。
   */
  serviceId: string;
  /**
   * 环境名称，当前支持三个环境，test（测试环境）、prepub（预发布环境）和 release（发布环境）。
   */
  environment: ApigwEnviromentType;
  /** 要部署的 API 网关 API 列表 */
  apiList: ApigwEndpoint[];
  /** 已部署的 API 网关 API 列表 */
  oldList: ApigwEndpoint[];
  /** API 网关 API */
  apiConfig: ApigwEndpoint;
  /** 是否是 OAUTH API */
  isOauthApi?: boolean;
}

/** 创建 API 网关输入 */
export interface ApigwDeployInputs
  extends ApigwCreateOrUpdateServiceInputs,
    ApigwBindCustomDomainInputs {
  /** 地域 */
  region?: RegionType;
  /** 上一次部署后状态 */
  oldState?: any;
  /** API 网关发布环境类型 */
  environment?: ApigwEnviromentType;
  /** API 网关 API 列表 */
  endpoints: ApigwEndpoint[];
}

/** 绑定 API 网关自定义域名输出 */
export interface ApigwBindCustomDomainOutputs {
  /** 是否绑定 */
  isBinded: boolean;
  /** 是否创建 */
  created?: boolean;
  /** 域名 */
  subDomain: string;
  /** 域名 CNAME */
  cname: string;
  /** 域名 URL */
  url?: string;
  /** 输出信息 */
  message?: string;
}

/** 绑定用户计划输出 */
export interface ApigwUsagePlanOutputs {
  /** 是否创建 */
  created?: boolean;
  /** 创建后的用户计划 ID */
  usagePlanId: string;
}

/** API 网关部署输出 */
export interface ApigwDeployOutputs {
  /** 是否创建 */
  created?: boolean;
  /**
   * 服务的唯一ID。
   */
  serviceId?: string;
  /**
   * API所在服务名称。
   */
  serviceName: string;
  /** 访问域名 */
  subDomain?: string | string[];
  /** 协议 */
  protocols: string | ('http' | 'https')[];
  /** API 网关发布环境类型 */
  environment: ApigwEnviromentType;
  /** API 网关 API 列表 */
  apiList: ApigwEndpoint[];
  /** 绑定 API 网关自定义域名输出 */
  customDomains?: ApigwBindCustomDomainOutputs[];
  /** 绑定用户计划输出 */
  usagePlan?: ApigwUsagePlanOutputs;
}

/** API 网关移除或解绑用户计划输入 */
export interface ApigwRemoveOrUnbindUsagePlanInputs {
  /**
   * 服务的唯一ID。
   */
  serviceId?: string;
  /** API 网关发布环境类型 */
  environment: ApigwEnviromentType;
  /** API 网关用量计划 */
  usagePlan: ApigwUsagePlan;
  /**
   * API 接口唯一 ID。
   */
  apiId?: string;
}

/** API 网关移除 API 输入 */
export interface ApigwApiRemoverInputs {
  /** API 网关 API */
  apiConfig: ApigwEndpoint;
  /**
   * 服务的唯一ID。
   */
  serviceId: string;
  /** API 网关发布环境类型 */
  environment: ApigwEnviromentType;
}

/** API 网关移除输入 */
export interface ApigwRemoveInputs {
  /** 是否被创建，记录用 */
  created?: boolean;
  /** API 网关发布环境类型 */
  environment: ApigwEnviromentType;
  /**
   * 服务的唯一ID。
   */
  serviceId?: string;
  /** API 网关 API 列表 */
  apiList: ApigwEndpoint[];
  /** 绑定 API 网关自定义域名输出 */
  customDomains?: ApigwBindCustomDomainOutputs[];
  /** API 网关用量计划 */
  usagePlan?: ApigwUsagePlan;
}

/** API 网关移除触发器输入 */
export interface ApigwTriggerRemoveInputs {
  /**
   * 服务的唯一ID。
   */
  serviceId: string;
  /**
   * API 接口唯一 ID。
   */
  apiId: string;
}

/** API 网关移除触发器输入 */
export interface ApigwTriggerRemoveScfTriggerInputs {
  /**
   * 服务的唯一ID。
   */
  serviceId: string;
  /**
   * API 接口唯一 ID。
   */
  apiId: string;
  /** 函数名 */
  functionName: string;
  /** 函数名称空间 */
  namespace: string;
  /** 函数版本 */
  qualifier: string;
}
